'use strict';

var svg = require('virtual-hyperscript-svg');
var roundTo = require('round-precision');
var sl = require('sainte-lague');

var pi = Math.PI;

var round = function round(x) {
	return roundTo(x, 10);
};
var seatSum = function seatSum(o) {
	var result = 0;
	for (var key in o) {
		result += o[key].seats;
	}return result;
};
var merge = function merge(arrays) {
	var result = [];
	var _iteratorNormalCompletion = true;
	var _didIteratorError = false;
	var _iteratorError = undefined;

	try {
		for (var _iterator = arrays[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
			var list = _step.value;
			result = result.concat(list);
		}
	} catch (err) {
		_didIteratorError = true;
		_iteratorError = err;
	} finally {
		try {
			if (!_iteratorNormalCompletion && _iterator.return) {
				_iterator.return();
			}
		} finally {
			if (_didIteratorError) {
				throw _iteratorError;
			}
		}
	}

	return result;
};

var coords = function coords(r, b) {
	return {
		x: round(r * Math.cos(b / r - pi)),
		y: round(r * Math.sin(b / r - pi))
	};
};

var calculateSeatDistance = function calculateSeatDistance(seatCount, numberOfRings, r) {
	var x = pi * numberOfRings * r / (seatCount - numberOfRings);
	var y = 1 + pi * (numberOfRings - 1) * numberOfRings / 2 / (seatCount - numberOfRings);

	var a = x / y;
	return a;
};

var score = function score(m, n, r) {
	return Math.abs(calculateSeatDistance(m, n, r) * n / r - 5 / 7);
};

var calculateNumberOfRings = function calculateNumberOfRings(seatCount, r) {
	var n = Math.floor(Math.log(seatCount) / Math.log(2)) || 1;
	var distance = score(seatCount, n, r);

	var direction = 0;
	if (score(seatCount, n + 1, r) < distance) direction = 1;
	if (score(seatCount, n - 1, r) < distance && n > 1) direction = -1;

	while (score(seatCount, n + direction, r) < distance && n > 0) {
		distance = score(seatCount, n + direction, r);
		n += direction;
	}
	return n;
};

var nextRing = function nextRing(rings, ringProgress) {
	var progressQuota = void 0,
	    tQuota = void 0;
	for (var index in rings) {
		tQuota = round((ringProgress[index] || 0) / rings[index].length);
		if (!progressQuota || tQuota < progressQuota) progressQuota = tQuota;
	}
	for (var _index in rings) {
		tQuota = round((ringProgress[_index] || 0) / rings[_index].length);
		if (tQuota == progressQuota) return _index;
	}
};

var generatePoints = function generatePoints(parliament, r0) {

	var seatCount = seatSum(parliament);
	var numberOfRings = calculateNumberOfRings(seatCount, r0);
	var seatDistance = calculateSeatDistance(seatCount, numberOfRings, r0);

	// calculate ring radii
	var rings = [];
	for (var i = 1; i <= numberOfRings; i++) {
		rings[i] = r0 - (i - 1) * seatDistance;
	}

	// calculate seats per ring
	// todo: float to int
	rings = sl(rings, seatCount);

	var points = [];
	var r = void 0,
	    a = void 0,
	    point = void 0;

	// build seats
	// loop rings
	var ring = void 0;
	for (var _i = 1; _i <= numberOfRings; _i++) {
		ring = [];
		// calculate ring-specific radius
		r = r0 - (_i - 1) * seatDistance;
		// calculate ring-specific distance
		a = pi * r / (rings[_i] - 1 || 1);

		// loop points
		for (var j = 0; j <= rings[_i] - 1; j++) {
			point = coords(r, j * a);
			point.r = 0.4 * seatDistance;
			ring.push(point);
		}
		points.push(ring);
	}

	// fill seats
	var ringProgress = Array(points.length).fill(0);
	for (var party in parliament) {
		for (var _i2 = 0; _i2 < parliament[party].seats; _i2++) {
			ring = nextRing(points, ringProgress);
			points[ring][ringProgress[ring]].fill = parliament[party].colour;
			points[ring][ringProgress[ring]].party = party;
			ringProgress[ring]++;
		}
	}

	return merge(points);
};

var pointToSVG = function pointToSVG(point) {
	return svg('circle', {
		cx: point.x,
		cy: point.y,
		r: point.r,
		fill: point.fill,
		class: point.party
	});
};

var generate = function generate(parliament, seatCount) {
	var radius = 20;
	var points = generatePoints(parliament, radius);
	var a = points[0].r / 0.4;
	var elements = points.map(pointToSVG);
	if (seatCount) {
		elements.push(svg('text', {
			x: 0,
			y: 0,
			"text-anchor": 'middle',
			style: {
				'font-family': 'Helvetica',
				'font-size': 0.25 * radius + 'px'
			},
			class: 'seatNumber'
		}, elements.length));
	}
	var document = svg('svg', {
		xmlns: 'http://www.w3.org/2000/svg',
		viewBox: [-radius - a / 2, -radius - a / 2, 2 * radius + a, radius + a].join(',')
	}, elements);
	return document;
};

module.exports = generate;